import React, { useCallback, useEffect, useMemo, useState, } from "react";
import { DataGridPro, GridToolbarContainer, GridToolbarFilterButton, GridLinkOperator, } from "@mui/x-data-grid-pro";
import { withNetworkCompletionDispatch, } from "@nerdjs/nerd-network";
import StringOperatorInput from "./customOperatorInput/stringTypeInput";
import DateOperatorInput from "./customOperatorInput/dateTypeInput";
import BoolOperatorInput from "./customOperatorInput/boolTypeInput";
import CustomPagination from "../NerdPagination/NerdPagination";
import { Button } from "@mui/material";
import { SaveAlt } from "@mui/icons-material";
const OPERATORS = {
    eq: {
        label: "Equal",
        value: "eq",
    },
    neq: {
        label: "Not Equal",
        value: "neq",
    },
    gt: {
        label: "Greater Than",
        value: "gt",
    },
    lt: {
        label: "Less Than",
        value: "lt",
    },
    gte: {
        label: "Greater or Equal Than",
        value: "gte",
    },
    lte: {
        label: "Less or Equal Than",
        value: "lte",
    },
    like: {
        label: "Like",
        value: "like",
    },
};
/**
 * NerdGrid component.
 *
 * @param {propsType} props nerdGrid props.
 * @returns {React.ReactElement} DataGrid wrapper with server-side filters.
 */
function NerdGrid(props) {
    const { columnsVisibility, columns, networkCompletionStack, dispatch, queryBuilder, getRows, culumnsMatchingFilterName, hidePagination, hideFilters, defaultSelectRowID, createNewButton, } = props;
    const rows = props.rows;
    const rowPageOptions = useMemo(() => [10, 50, 100], []);
    let limit;
    let offset;
    let filters;
    if (queryBuilder) {
        const queryParams = queryBuilder.getQuery();
        limit = queryParams["limit"];
        offset = queryParams["offset"];
        filters = queryParams["filters"];
    }
    const [rowsPerPage, setRowsPerPage] = useState(10);
    const [page, setPage] = useState(1);
    const [selectPages, setSelectPages] = useState([0]);
    const [disabledNext, setDisabledNext] = useState(false);
    const [rowCount, setRowCount] = React.useState(undefined);
    const [rowCountState, setRowCountState] = React.useState(rowCount || 0);
    const [queryObject, setQueryObject] = useState(culumnsMatchingFilterName
        ? {
            filters: filters ? filters : "",
            limit: Number(limit) ? Number(limit) : rowsPerPage,
            offset: Number(offset) ? Number(offset) : 0,
        }
        : {
            limit: Number(limit) ? Number(limit) : rowsPerPage,
            offset: Number(offset) ? Number(offset) : 0,
        });
    const [requestPending, setRequestPending] = useState(false);
    const [requestUUID, setRequestUUID] = useState("");
    const [filterModel, setFilterModel] = useState({
        items: [],
        linkOperator: GridLinkOperator.And,
    });
    const [selectionModel, setSelectionModel] = useState(defaultSelectRowID &&
        rows.filter((row) => row["id"] === defaultSelectRowID).length > 0
        ? [defaultSelectRowID]
        : []);
    const prevSelectionModel = React.useRef(defaultSelectRowID ? [defaultSelectRowID] : []);
    /**
     * Custom MUI Grid Toolbar
     *
     * @returns {React.ReactElement} Mui Grid Toolbar
     */
    function CustomToolbar() {
        return (React.createElement(GridToolbarContainer, null,
            !hideFilters && React.createElement(GridToolbarFilterButton, null),
            props.apiExportEndpoint && (React.createElement(Button, { sx: { fontSize: "0.8125rem" }, variant: "text", startIcon: React.createElement(SaveAlt, null), onClick: () => {
                    const a = encodeURI(`${props.apiExportEndpoint}${"filters" in queryObject
                        ? `?filters=${queryObject.filters}`
                        : ""}`);
                    window.open(a, "_blank", "noopener,noreferrer");
                } }, "EXPORT")),
            createNewButton));
    }
    useEffect(() => {
        try {
            if (culumnsMatchingFilterName && filters) {
                const filtersObject = JSON.parse(filters);
                const filterModelTmp = {
                    items: [],
                    linkOperator: GridLinkOperator.And,
                };
                Array.isArray(filtersObject) &&
                    filtersObject.forEach((filter, i) => {
                        let columnField = "";
                        for (const [key, value] of culumnsMatchingFilterName.entries()) {
                            if (value === filter.name)
                                columnField = key;
                        }
                        filterModelTmp.items.push({
                            id: i + 1,
                            columnField,
                            value: filter.value,
                            operatorValue: filter.comparison,
                        });
                    });
                setFilterModel(filterModelTmp);
            }
        }
        catch (error) {
            return;
        }
    }, []);
    // Set URL query
    useEffect(() => {
        if (queryBuilder) {
            if (Number(offset) % Number(limit) !== 0 ||
                !rowPageOptions.includes(Number(limit))) {
                setQueryObject(Object.assign(Object.assign({}, queryObject), { limit: 10, offset: 0 }));
                setPage(1);
                setRowsPerPage(10);
            }
            else if (Number(limit) === queryObject.limit) {
                setPage(Number(offset) / Number(limit) + 1);
                setRowsPerPage(Number(limit));
            }
            queryBuilder.setUrlQuery(queryObject);
        }
    }, [offset, limit, rowPageOptions, queryObject, queryBuilder]);
    /**
     * Return DataGrid filter operators for a given column type.
     *
     * @param {string | undefined} type Type of the column.
     * @returns {Array<GridFilterOperator>} Return filter operators.
     */
    function operatorsByType(type) {
        const operators = [];
        let input;
        switch (type) {
            case "date":
                input = DateOperatorInput;
                break;
            case "boolean":
                input = BoolOperatorInput;
                break;
            default:
                input = StringOperatorInput;
                break;
        }
        Object.entries(OPERATORS).forEach((val) => operators.push({
            label: val[1].label,
            value: val[1].value,
            getApplyFilterFn: () => {
                //ServerSide filter, so we don't need filter function here.
                return () => {
                    return true;
                };
            },
            InputComponent: input,
        }));
        return operators;
    }
    const setCustomPagination = useCallback(() => {
        if (Number(offset) > 0 || Number(limit) > 0) {
            setSelectPages((prevState) => [
                ...Array.from(new Set(prevState)),
                ...Array(page)
                    .fill(0)
                    .map((_e, i) => i + 1),
            ]);
        }
    }, [limit, offset, page]);
    const columnsWithOperators = React.useMemo(() => columns.map((col) => {
        return Object.assign(Object.assign({}, col), { filterOperators: operatorsByType(col.type) });
    }), [columns]);
    useEffect(() => {
        if (dispatch && getRows) {
            setRequestPending(true);
            setRowCount(undefined);
            const uuid = withNetworkCompletionDispatch(dispatch, getRows(queryObject));
            setRequestUUID(uuid);
        }
    }, [dispatch, getRows, queryObject]);
    // Check if API call ends and close loading
    useEffect(() => {
        if (networkCompletionStack) {
            if (requestUUID !== "" &&
                !networkCompletionStack.some((e) => e[requestUUID])) {
                setRequestUUID("");
                setRequestPending(false);
                setRowCount(rows.length);
                //restore selection model after page change
                setSelectionModel(prevSelectionModel.current);
            }
            setCustomPagination();
        }
    }, [requestUUID, networkCompletionStack, setCustomPagination, rows.length]);
    const onFilterChange = React.useCallback((newFilterModel) => {
        if (culumnsMatchingFilterName) {
            const filters = [];
            newFilterModel.items.forEach((item) => {
                const searchMetadataName = culumnsMatchingFilterName.get(item.columnField);
                if (item.operatorValue &&
                    item.value !== undefined &&
                    item.value !== "" &&
                    searchMetadataName) {
                    filters.push({
                        name: searchMetadataName,
                        comparison: item.operatorValue,
                        value: item.value,
                    });
                }
            });
            setPage(1); // Reset page to 1
            setSelectPages([0]); // Reset select pages
            setQueryObject({
                limit: queryObject.limit,
                offset: 0,
                filters: JSON.stringify(filters),
            });
            setFilterModel(newFilterModel);
        }
    }, [culumnsMatchingFilterName, queryObject.limit]);
    /**
     * Set new page value and change the query object
     *
     * @param {React.MouseEvent<HTMLButtonElement> | null} _event ChangePage
     * @param {number} newPage New page index
     */
    function handleChangePage(_event, newPage) {
        prevSelectionModel.current = selectionModel;
        setPage(newPage);
        if (newPage > 0) {
            setQueryObject(Object.assign(Object.assign({}, queryObject), { offset: (newPage - 1) * Number(limit) }));
        }
    }
    /**
     * @param {ChangeEvent<HTMLInputElement | HTMLTextAreaElement>} event - new rows page number selected
     */
    function handleChangeRowsPerPage(event) {
        const newPage = parseInt(event.target.value, 10);
        setRowsPerPage(newPage); // set new page size
        setPage(1); // Reset page to 1
        setSelectPages([0]); // Reset select pages
        // Set new query object
        setQueryObject((prevState) => (Object.assign(Object.assign({}, prevState), { limit: newPage, offset: 0 })));
    }
    /**
     *
     * @param {GridSelectionModel} ids Selected rows
     */
    function onSelectionModelChange(ids) {
        var _a, _b;
        const selectedIDs = new Set(ids);
        const selectedRows = rows.filter((row) => selectedIDs.has(row.id));
        const prevSelectedIDs = new Set(prevSelectionModel.current);
        //Don't call onSelect if page changed
        if (ids.length === 0) {
            if (rows.filter((row) => prevSelectedIDs.has(row.id)).length > 0) {
                (_a = props.onSelect) === null || _a === void 0 ? void 0 : _a.call(props, selectedRows);
            }
        }
        else {
            (_b = props.onSelect) === null || _b === void 0 ? void 0 : _b.call(props, selectedRows);
        }
        setSelectionModel(ids);
    }
    React.useEffect(() => {
        setRowCountState((prevRowCountState) => rowCount !== undefined ? rowCount : prevRowCountState);
    }, [rowCount, setRowCountState]);
    return (React.createElement(DataGridPro, { sx: {
            "&.MuiDataGrid-root .MuiDataGrid-cell:focus": {
                outline: "none",
            },
            paddingTop: "0px",
            width: "100%",
            height: "700px",
        }, pageSize: rowsPerPage, rowsPerPageOptions: rowPageOptions, onStateChange: (state) => {
            const { pagination: { rowCount, pageSize }, } = state;
            setDisabledNext(rowCount < pageSize);
        }, pagination: true, paginationMode: "server", rowCount: rowCountState, hideFooterPagination: hidePagination, components: {
            Toolbar: CustomToolbar,
            Pagination: CustomPagination,
        }, componentsProps: {
            filterPanel: { linkOperators: ["and"] },
            pagination: {
                rowsPerPage,
                handleChangeRowsPerPage,
                rowPageOptions,
                handleChangePage,
                page,
                selectPages,
                disabledNext,
            },
        }, onSelectionModelChange: onSelectionModelChange, rows: rows, columns: columnsWithOperators, loading: requestPending, filterMode: "server", onFilterModelChange: onFilterChange, initialState: {
            columns: {
                columnVisibilityModel: columnsVisibility,
            },
        }, filterModel: filterModel, selectionModel: selectionModel }));
}
export default NerdGrid;
